@@ -32,17 +32,17 @@
  *
  * Applications can set a context on a complete pipeline by using
  * gst_element_set_context(), which will then be propagated to all
- * child elements. Elements can handle these in GstElement::set_context()
+ * child elements. Elements can handle these in #GstElementClass.set_context()
  * and merge them with the context information they already have.
  *
  * When an element needs a context it will do the following actions in this
  * order until one step succeeds:
- * 1) Check if the element already has a context
- * 2) Query downstream with GST_QUERY_CONTEXT for the context
- * 2) Query upstream with GST_QUERY_CONTEXT for the context
- * 3) Post a GST_MESSAGE_NEED_CONTEXT message on the bus with the required
+ * 1. Check if the element already has a context
+ * 2. Query downstream with GST_QUERY_CONTEXT for the context
+ * 3. Query upstream with GST_QUERY_CONTEXT for the context
+ * 4. Post a GST_MESSAGE_NEED_CONTEXT message on the bus with the required
  *    context types and afterwards check if a usable context was set now
- * 4) Create a context by itself and post a GST_MESSAGE_HAVE_CONTEXT message
+ * 5. Create a context by itself and post a GST_MESSAGE_HAVE_CONTEXT message
  *    on the bus.
  *
  * Bins will catch GST_MESSAGE_NEED_CONTEXT messages and will set any previously
@@ -68,7 +68,7 @@ struct _GstContext
 
 #define GST_CONTEXT_STRUCTURE(c)  (((GstContext *)(c))->structure)
 
-static GType _gst_context_type = 0;
+GType _gst_context_type = 0;
 GST_DEFINE_MINI_OBJECT_TYPE (GstContext, gst_context);
 
 void
@@ -141,6 +141,7 @@ gst_context_init (GstContext * context)
 
 /**
  * gst_context_new:
+ * @context_type: Context type
  * @persistent: Persistent context
  *
  * Create a new context.
@@ -205,8 +206,8 @@ gboolean
 gst_context_has_context_type (const GstContext * context,
     const gchar * context_type)
 {
-  g_return_val_if_fail (GST_IS_CONTEXT (context), NULL);
-  g_return_val_if_fail (context_type != NULL, NULL);
+  g_return_val_if_fail (GST_IS_CONTEXT (context), FALSE);
+  g_return_val_if_fail (context_type != NULL, FALSE);
 
   return strcmp (context->context_type, context_type) == 0;
 }
@@ -266,7 +267,7 @@ gst_context_writable_structure (GstContext * context)
 gboolean
 gst_context_is_persistent (const GstContext * context)
 {
-  g_return_val_if_fail (GST_IS_CONTEXT (context), NULL);
+  g_return_val_if_fail (GST_IS_CONTEXT (context), FALSE);
 
   return context->persistent;
 }
