@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 
@@ -81,6 +81,7 @@ struct _GstSystemClockClass {
 GType                   gst_system_clock_get_type       (void);
 
 GstClock*               gst_system_clock_obtain         (void);
+void                    gst_system_clock_set_default    (GstClock *new_clock);
 
 G_END_DECLS
 
