@@ -25,9 +25,8 @@
  *
  * List of tags and values used to describe media metadata.
  *
- * Strings must be in ASCII or UTF-8 encoding. No other encodings are allowed.
- *
- * Last reviewed on 2009-06-09 (0.10.23)
+ * Strings in structures must be ASCII or UTF-8 encoded. Other encodings are
+ * not allowed. Strings must not be empty or %NULL.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -85,6 +84,7 @@ static GMutex __tag_mutex;
 /* tags hash table: maps tag name string => GstTagInfo */
 static GHashTable *__tags;
 
+GType _gst_tag_list_type = 0;
 GST_DEFINE_MINI_OBJECT_TYPE (GstTagList, gst_tag_list);
 
 static void __gst_tag_list_free (GstTagList * list);
@@ -99,6 +99,8 @@ _priv_gst_tag_initialize (void)
 {
   g_mutex_init (&__tag_mutex);
 
+  _gst_tag_list_type = gst_tag_list_get_type ();
+
   __tags = g_hash_table_new (g_str_hash, g_str_equal);
   gst_tag_register_static (GST_TAG_TITLE, GST_TAG_FLAG_META,
       G_TYPE_STRING,
@@ -387,6 +389,18 @@ _priv_gst_tag_initialize (void)
   gst_tag_register_static (GST_TAG_IMAGE_ORIENTATION, GST_TAG_FLAG_META,
       G_TYPE_STRING, _("image orientation"),
       _("How the image should be rotated or flipped before display"), NULL);
+  gst_tag_register_static (GST_TAG_PUBLISHER, GST_TAG_FLAG_META,
+      G_TYPE_STRING,
+      _("publisher"),
+      _("Name of the label or publisher"), gst_tag_merge_strings_with_comma);
+  gst_tag_register_static (GST_TAG_INTERPRETED_BY, GST_TAG_FLAG_META,
+      G_TYPE_STRING,
+      _("interpreted-by"),
+      _("Information about the people behind a remix and similar "
+          "interpretations"), gst_tag_merge_strings_with_comma);
+  gst_tag_register_static (GST_TAG_MIDI_BASE_NOTE, GST_TAG_FLAG_META,
+      G_TYPE_UINT,
+      _("midi-base-note"), _("Midi note number of the audio track."), NULL);
 }
 
 /**
@@ -454,7 +468,7 @@ gst_tag_lookup (const gchar * tag_name)
  * @type: the type this data is in
  * @nick: human-readable name
  * @blurb: a human-readable description about this tag
- * @func: (allow-none): function for merging multiple values of this tag, or NULL
+ * @func: (allow-none): function for merging multiple values of this tag, or %NULL
  *
  * Registers a new tag type for the use with GStreamer's type system. If a type
  * with that name is already registered, that one is used.
@@ -463,7 +477,7 @@ gst_tag_lookup (const gchar * tag_name)
  *
  * Important: if you do not supply a merge function the implication will be
  * that there can only be one single value for this tag in a tag list and
- * any additional values will silenty be discarded when being added (unless
+ * any additional values will silently be discarded when being added (unless
  * #GST_TAG_MERGE_REPLACE, #GST_TAG_MERGE_REPLACE_ALL, or
  * #GST_TAG_MERGE_PREPEND is used as merge mode, in which case the new
  * value will replace the old one in the list).
@@ -498,7 +512,7 @@ gst_tag_register (const gchar * name, GstTagFlag flag, GType type,
  * @type: the type this data is in
  * @nick: human-readable name or short description (string constant)
  * @blurb: a human-readable description for this tag (string constant)
- * @func: (allow-none): function for merging multiple values of this tag, or NULL
+ * @func: (allow-none): function for merging multiple values of this tag, or %NULL
  *
  * Registers a new tag type for the use with GStreamer's type system.
  *
@@ -544,7 +558,7 @@ gst_tag_register_static (const gchar * name, GstTagFlag flag, GType type,
  *
  * Checks if the given type is already registered.
  *
- * Returns: TRUE if the type is already registered
+ * Returns: %TRUE if the type is already registered
  */
 gboolean
 gst_tag_exists (const gchar * tag)
@@ -643,7 +657,7 @@ gst_tag_get_flag (const gchar * tag)
  * Checks if the given tag is fixed. A fixed tag can only contain one value.
  * Unfixed tags can contain lists of values.
  *
- * Returns: TRUE, if the given tag is fixed.
+ * Returns: %TRUE, if the given tag is fixed.
  */
 gboolean
 gst_tag_is_fixed (const gchar * tag)
@@ -729,10 +743,10 @@ gst_tag_list_new_empty (void)
 /**
  * gst_tag_list_new:
  * @tag: tag
- * @...: NULL-terminated list of values to set
+ * @...: %NULL-terminated list of values to set
  *
  * Creates a new taglist and appends the values for the given tags. It expects
- * tag-value pairs like gst_tag_list_add(), and a NULL terminator after the
+ * tag-value pairs like gst_tag_list_add(), and a %NULL terminator after the
  * last pair. The type of the values is implicit and is documented in the API
  * reference, but can also be queried at runtime with gst_tag_get_type(). It
  * is an error to pass a value of a type not matching the tag type into this
@@ -830,8 +844,9 @@ gst_tag_list_get_scope (const GstTagList * list)
  *
  * Serializes a tag list to a string.
  *
- * Returns: a newly-allocated string, or NULL in case of an error. The
- *    string must be freed with g_free() when no longer needed.
+ * Returns: (nullable): a newly-allocated string, or %NULL in case of
+ *     an error. The string must be freed with g_free() when no longer
+ *     needed.
  */
 gchar *
 gst_tag_list_to_string (const GstTagList * list)
@@ -847,7 +862,8 @@ gst_tag_list_to_string (const GstTagList * list)
  *
  * Deserializes a tag list.
  *
- * Returns: a new #GstTagList, or NULL in case of an error.
+ * Returns: (nullable): a new #GstTagList, or %NULL in case of an
+ * error.
  */
 GstTagList *
 gst_tag_list_new_from_string (const gchar * str)
@@ -908,7 +924,7 @@ gst_tag_list_nth_tag_name (const GstTagList * list, guint index)
  *
  * Checks if the given taglist is empty.
  *
- * Returns: TRUE if the taglist is empty, otherwise FALSE.
+ * Returns: %TRUE if the taglist is empty, otherwise %FALSE.
  */
 gboolean
 gst_tag_list_is_empty (const GstTagList * list)
@@ -947,7 +963,7 @@ gst_tag_list_fields_equal (const GValue * value1, const GValue * value2)
  *
  * Checks if the two given taglists are equal.
  *
- * Returns: TRUE if the taglists are equal, otherwise FALSE
+ * Returns: %TRUE if the taglists are equal, otherwise %FALSE
  */
 gboolean
 gst_tag_list_is_equal (const GstTagList * list1, const GstTagList * list2)
@@ -1117,16 +1133,16 @@ gst_tag_list_insert (GstTagList * into, const GstTagList * from,
 
 /**
  * gst_tag_list_merge:
- * @list1: first list to merge
- * @list2: second list to merge
+ * @list1: (allow-none): first list to merge
+ * @list2: (allow-none): second list to merge
  * @mode: the mode to use
  *
- * Merges the two given lists into a new list. If one of the lists is NULL, a
- * copy of the other is returned. If both lists are NULL, NULL is returned.
+ * Merges the two given lists into a new list. If one of the lists is %NULL, a
+ * copy of the other is returned. If both lists are %NULL, %NULL is returned.
  *
  * Free-function: gst_tag_list_unref
  *
- * Returns: (transfer full): the new list
+ * Returns: (transfer full) (nullable): the new list
  */
 GstTagList *
 gst_tag_list_merge (const GstTagList * list1, const GstTagList * list2,
@@ -1186,7 +1202,7 @@ gst_tag_list_get_tag_size (const GstTagList * list, const gchar * tag)
  * @list: list to set tags in
  * @mode: the mode to use
  * @tag: tag
- * @...: NULL-terminated list of values to set
+ * @...: %NULL-terminated list of values to set
  *
  * Sets the values for the given tags using the specified mode.
  */
@@ -1410,8 +1426,9 @@ gst_tag_list_foreach (const GstTagList * list, GstTagForeachFunc func,
  * Gets the value that is at the given index for the given tag in the given
  * list.
  *
- * Returns: (transfer none): The GValue for the specified entry or NULL if the
- *          tag wasn't available or the tag doesn't have as many entries
+ * Returns: (transfer none) (nullable): The GValue for the specified
+ *          entry or %NULL if the tag wasn't available or the tag
+ *          doesn't have as many entries
  */
 const GValue *
 gst_tag_list_get_value_index (const GstTagList * list, const gchar * tag,
@@ -1448,7 +1465,7 @@ gst_tag_list_get_value_index (const GstTagList * list, const gchar * tag,
  * with the tag.
  * You must g_value_unset() the value after use.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *          given list.
  */
 gboolean
@@ -1538,7 +1555,7 @@ gst_tag_list_get_ ## name ## _index (const GstTagList *list,            \
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 /**
@@ -1551,7 +1568,7 @@ gst_tag_list_get_ ## name ## _index (const GstTagList *list,            \
  * Gets the value that is at the given index for the given tag in the given
  * list.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 TAG_MERGE_FUNCS (boolean, gboolean, TRUE);
@@ -1564,7 +1581,7 @@ TAG_MERGE_FUNCS (boolean, gboolean, TRUE);
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 /**
@@ -1577,7 +1594,7 @@ TAG_MERGE_FUNCS (boolean, gboolean, TRUE);
  * Gets the value that is at the given index for the given tag in the given
  * list.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 TAG_MERGE_FUNCS (int, gint, TRUE);
@@ -1590,7 +1607,7 @@ TAG_MERGE_FUNCS (int, gint, TRUE);
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 /**
@@ -1603,7 +1620,7 @@ TAG_MERGE_FUNCS (int, gint, TRUE);
  * Gets the value that is at the given index for the given tag in the given
  * list.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 TAG_MERGE_FUNCS (uint, guint, TRUE);
@@ -1617,7 +1634,7 @@ TAG_MERGE_FUNCS (uint, guint, TRUE);
  * Gets the value that is at the given index for the given tag in the given
  * list.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 TAG_MERGE_FUNCS (int64, gint64, TRUE);
@@ -1630,7 +1647,7 @@ TAG_MERGE_FUNCS (int64, gint64, TRUE);
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 /**
@@ -1643,7 +1660,7 @@ TAG_MERGE_FUNCS (int64, gint64, TRUE);
  * Gets the value that is at the given index for the given tag in the given
  * list.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 TAG_MERGE_FUNCS (uint64, guint64, TRUE);
@@ -1656,7 +1673,7 @@ TAG_MERGE_FUNCS (uint64, guint64, TRUE);
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 /**
@@ -1669,7 +1686,7 @@ TAG_MERGE_FUNCS (uint64, guint64, TRUE);
  * Gets the value that is at the given index for the given tag in the given
  * list.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 TAG_MERGE_FUNCS (float, gfloat, TRUE);
@@ -1682,7 +1699,7 @@ TAG_MERGE_FUNCS (float, gfloat, TRUE);
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 /**
@@ -1695,7 +1712,7 @@ TAG_MERGE_FUNCS (float, gfloat, TRUE);
  * Gets the value that is at the given index for the given tag in the given
  * list.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 TAG_MERGE_FUNCS (double, gdouble, TRUE);
@@ -1708,7 +1725,7 @@ TAG_MERGE_FUNCS (double, gdouble, TRUE);
  * Copies the contents for the given tag into the value, merging multiple values
  * into one if multiple values are associated with the tag.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 /**
@@ -1721,7 +1738,7 @@ TAG_MERGE_FUNCS (double, gdouble, TRUE);
  * Gets the value that is at the given index for the given tag in the given
  * list.
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 TAG_MERGE_FUNCS (pointer, gpointer, (*value != NULL));
@@ -1752,11 +1769,11 @@ _gst_strdup0 (const gchar * s)
  *
  * The resulting string in @value will be in UTF-8 encoding and should be
  * freed by the caller using g_free when no longer needed. The
- * returned string is also guaranteed to be non-NULL and non-empty.
+ * returned string is also guaranteed to be non-%NULL and non-empty.
  *
  * Free-function: g_free
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 /**
@@ -1771,11 +1788,11 @@ _gst_strdup0 (const gchar * s)
  *
  * The resulting string in @value will be in UTF-8 encoding and should be
  * freed by the caller using g_free when no longer needed. The
- * returned string is also guaranteed to be non-NULL and non-empty.
+ * returned string is also guaranteed to be non-%NULL and non-empty.
  *
  * Free-function: g_free
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
  *              given list.
  */
 TAG_MERGE_FUNCS (string, gchar *, (*value != NULL));
@@ -1797,9 +1814,9 @@ TAG_MERGE_FUNCS (string, gchar *, (*value != NULL));
  *
  * The resulting string in @value will be in UTF-8 encoding and doesn't need
  * to be freed by the caller. The returned string is also guaranteed to
- * be non-NULL and non-empty.
+ * be non-%NULL and non-empty.
  *
- * Returns: TRUE, if a value was set, FALSE if the tag didn't exist in the
+ * Returns: %TRUE, if a value was set, %FALSE if the tag didn't exist in the
  *              given list.
  */
 gboolean
@@ -1831,8 +1848,8 @@ gst_tag_list_peek_string_index (const GstTagList * list,
  *
  * Free-function: g_date_free
  *
- * Returns: TRUE, if a date was copied, FALSE if the tag didn't exist in the
- *              given list or if it was #NULL.
+ * Returns: %TRUE, if a date was copied, %FALSE if the tag didn't exist in the
+ *              given list or if it was %NULL.
  */
 gboolean
 gst_tag_list_get_date (const GstTagList * list, const gchar * tag,
@@ -1864,8 +1881,8 @@ gst_tag_list_get_date (const GstTagList * list, const gchar * tag,
  *
  * Free-function: g_date_free
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
- *              given list or if it was #NULL.
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
+ *              given list or if it was %NULL.
  */
 gboolean
 gst_tag_list_get_date_index (const GstTagList * list,
@@ -1896,8 +1913,8 @@ gst_tag_list_get_date_index (const GstTagList * list,
  *
  * Free-function: gst_date_time_unref
  *
- * Returns: TRUE, if a datetime was copied, FALSE if the tag didn't exist in
- *              thegiven list or if it was #NULL.
+ * Returns: %TRUE, if a datetime was copied, %FALSE if the tag didn't exist in
+ *              the given list or if it was %NULL.
  */
 gboolean
 gst_tag_list_get_date_time (const GstTagList * list, const gchar * tag,
@@ -1912,8 +1929,6 @@ gst_tag_list_get_date_time (const GstTagList * list, const gchar * tag,
   if (!gst_tag_list_copy_value (&v, list, tag))
     return FALSE;
 
-  g_return_val_if_fail (GST_VALUE_HOLDS_DATE_TIME (&v), FALSE);
-
   *value = (GstDateTime *) g_value_dup_boxed (&v);
   g_value_unset (&v);
   return (*value != NULL);
@@ -1932,8 +1947,8 @@ gst_tag_list_get_date_time (const GstTagList * list, const gchar * tag,
  *
  * Free-function: gst_date_time_unref
  *
- * Returns: TRUE, if a value was copied, FALSE if the tag didn't exist in the
- *              given list or if it was #NULL.
+ * Returns: %TRUE, if a value was copied, %FALSE if the tag didn't exist in the
+ *              given list or if it was %NULL.
  */
 gboolean
 gst_tag_list_get_date_time_index (const GstTagList * list,
@@ -1966,8 +1981,8 @@ gst_tag_list_get_date_time_index (const GstTagList * list,
  *
  * Free-function: gst_sample_unref
  *
- * Returns: TRUE, if a sample was returned, FALSE if the tag didn't exist in
- *              the given list or if it was #NULL.
+ * Returns: %TRUE, if a sample was returned, %FALSE if the tag didn't exist in
+ *              the given list or if it was %NULL.
  */
 gboolean
 gst_tag_list_get_sample (const GstTagList * list, const gchar * tag,
@@ -1995,15 +2010,15 @@ gst_tag_list_get_sample (const GstTagList * list, const gchar * tag,
  *     pointer variable to store the result into
  *
  * Gets the sample that is at the given index for the given tag in the given
- * list and copies it into the variable pointed to by @smple. Free the sample
+ * list and copies it into the variable pointed to by @sample. Free the sample
  * with gst_sample_unref() when it is no longer needed. You can retrieve the
  * buffer from the sample using gst_sample_get_buffer() and the associated
  * caps (if any) with gst_sample_get_caps().
  *
  * Free-function: gst_sample_unref
  *
- * Returns: TRUE, if a sample was copied, FALSE if the tag didn't exist in the
- *              given list or if it was #NULL.
+ * Returns: %TRUE, if a sample was copied, %FALSE if the tag didn't exist in the
+ *              given list or if it was %NULL.
  */
 gboolean
 gst_tag_list_get_sample_index (const GstTagList * list,
